@@ -1,14 +1,15 @@
 """PyAsusWRT Library implementation"""
 
 # flake8: noqa
-from .asuswrt import (
+from .asuswrt import AsusWrtHttp as AsusWrtHttp
+from .exceptions import (
+    AsusWrtClientResponseError as AsusWrtClientResponseError,
     AsusWrtConnectionError as AsusWrtConnectionError,
     AsusWrtConnectionTimeoutError as AsusWrtConnectionTimeoutError,
     AsusWrtError as AsusWrtError,
-    AsusWrtHttp as AsusWrtHttp,
     AsusWrtLoginError as AsusWrtLoginError,
     AsusWrtResponseError as AsusWrtResponseError,
     AsusWrtValueError as AsusWrtValueError,
 )
 
-__version__ = "0.1.5"
+__version__ = "0.1.14"
