@@ -5,43 +5,66 @@ from __future__ import annotations
 import asyncio
 import base64
 from collections import namedtuple
+from dataclasses import dataclass
 from datetime import datetime
-import json
 import logging
 import math
-from typing import Any, Optional
+from typing import Any
 
 import aiohttp
 
-ASUSWRT_USR_AGENT = "asusrouter-Android-DUTUtil-1.0.0.245"
-ASUSWRT_ERROR_KEY = "error_status"
-ASUSWRT_TOKEN_KEY = "asus_token"
-ASUSWRT_LOGIN_PATH = "login.cgi"
-ASUSWRT_GET_PATH = "appGet.cgi"
-ASUSWRT_CMD_PATH = "applyapp.cgi"
+from .exceptions import (
+    AsusWrtClientResponseError,
+    AsusWrtConnectionError,
+    AsusWrtConnectionTimeoutError,
+    AsusWrtError,
+    AsusWrtLoginError,
+)
+from .helpers import (
+    _calculate_cpu_usage,
+    _get_json_result,
+    _parse_fw_info,
+    _parse_temperatures,
+)
 
-PARAM_APPOBJ = "appobj"
+_ASUSWRT_USR_AGENT = "asusrouter-Android-DUTUtil-1.0.0.245"
+_ASUSWRT_ERROR_KEY = "error_status"
+_ASUSWRT_ACTION_KEY = "action_mode"
+_ASUSWRT_HOOK_KEY = "hook"
+_ASUSWRT_TOKEN_KEY = "asus_token"
+_ASUSWRT_LOGIN_PATH = "login.cgi"
+_ASUSWRT_GET_PATH = "appGet.cgi"
+_ASUSWRT_CMD_PATH = "applyapp.cgi"
+_ASUSWRT_APPLY_PATH = "apply.cgi"
+_ASUSWRT_FW_PATH = "detect_firmware.asp"
+_ASUSWRT_TEMP_PATH = "ajax_coretmp.asp"
+_ASUSWRT_SVC_REQ = "rc_service"
+_ASUSWRT_SVC_REPLY = "run_service"
+_ASUSWRT_SVC_MODIFY = "modify"
 
-CMD_CLIENT_LIST = "get_clientlist"
-CMD_CPU_USAGE = "cpu_usage"
-CMD_DHCP_LEASE = "dhcpLeaseMacList"
-CMD_MEMORY_USAGE = "memory_usage"
-CMD_NET_TRAFFIC = "netdev"
-CMD_NVRAM = "nvram_get"
-CMD_UPTIME = "uptime"
-CMD_WAN_INFO = "wanlink"
+_CMD_CLIENT_LIST = "get_clientlist"
+_CMD_CPU_USAGE = "cpu_usage"
+_CMD_DHCP_LEASE = "dhcpLeaseMacList"
+_CMD_MEMORY_USAGE = "memory_usage"
+_CMD_NET_TRAFFIC = "netdev"
+_CMD_NVRAM = "nvram_get"
+_CMD_UPTIME = "uptime"
+_CMD_WAN_INFO = "wanlink"
+_CMD_REBOOT = "reboot"
+_CMD_LED_STATUS = "start_ctrl_led"
+_CMD_FW_CHECK = "firmware_check"
 
-DEFAULT_TIMEOUT = 5
-DEFAULT_HTTP_PORT = 80
-DEFAULT_HTTPS_PORT = 8443
+_PARAM_APPOBJ = "appobj"
 
-PROP_MAC_ADDR = "label_mac"
+_PROP_MAC_ADDR = "label_mac"
+_PROP_MODEL = "productid"
+_PROP_LED_STATUS = "led_val"
 
-NVRAM_INFO = [
+_NVRAM_INFO = [
     "acs_dfs",
     "model",
-    "productid",
-    PROP_MAC_ADDR,
+    _PROP_MODEL,
+    _PROP_MAC_ADDR,
     "buildinfo",
     "firmver",
     "firmver_org",
@@ -66,88 +89,90 @@ NVRAM_INFO = [
     "time_zone_dst",
     "time_zone_x",
     "time_zone_dstoff",
-    "time_zone",
     "ntp_server0",
 ]
 
-Device = namedtuple("Device", ["mac", "ip", "name", "node", "is_mesh_node", "is_wl"])
+_CACHE_SPECIFIC_URI = "specific_uri"
+
+DEFAULT_TIMEOUT = 5
+DEFAULT_HTTP_PORT = 80
+DEFAULT_HTTPS_PORT = 8443
+FW_CHECK_INTERVAL = 7200  # seconds, means 2 hour
+
+
+Device = namedtuple("Device", ["mac", "ip", "name", "node", "is_wl"])
 
 _LOGGER = logging.getLogger(__name__)
 
 
 def _nvram_cmd(info_type):
     """Return the cmd to get nvram data."""
-    return f"{CMD_NVRAM}({info_type})"
+    return f"{_CMD_NVRAM}({info_type})"
 
 
-def _get_json_result(result: str, json_key: str | None = None):
-    """Return the json result from a text result."""
-    try:
-        json_res = json.loads(result)
-    except json.JSONDecodeError as exc:
-        raise AsusWrtValueError(str(exc)) from exc
+@dataclass()
+class AsusWrtFirmware:
+    """Represent an AsusWrt firmware."""
 
-    if not json_key:
-        return json_res
+    version: str | None
+    build: str
+    extend: str | None
 
-    if (json_val := json_res.get(json_key)) is None:
-        raise AsusWrtValueError("No value available")
-    return json_val
+    def to_str(self) -> str | None:
+        """Convert firmware information to a readable string."""
+        retval = None
+        if self.version:
+            retval = self.version
+        if retval:
+            retval += f".{self.build}"
+        else:
+            retval = self.build
+        if self.extend:
+            retval += f"_{self.extend}"
+        return retval
+
+    def check_new(self, new_ver: str) -> str | None:
+        """Check if available fw differs from existing."""
+        chk_ver = None
+        if self.version:
+            chk_ver = self.version.replace(".", "")
+        chk_build = self.build.replace(".", "_")
+        if self.extend:
+            chk_build += f"_{self.extend}"
+        if new_ver == chk_build:
+            return None
+        if chk_ver:
+            if new_ver == f"{chk_ver}_{chk_build}":
+                return None
+        return new_ver
 
 
-class AsusWrtError(Exception):
-    """Base class for all errors raised by this library."""
+class AsusWrtCache:
+    """Class to manage command cache."""
 
-    def __init__(
-        self, *args: Any, message: Optional[str] = None, **_kwargs: Any
-    ) -> None:
-        """Initialize base AsusWrtError."""
-        super().__init__(*args, message)
+    def __init__(self, validity: int):
+        """Initialize the class."""
+        self._cache: dict = {}
+        self._validity = validity
 
+    def set_key(self, cache_type: str, key: str, value: Any) -> None:
+        """Add a value in the cache for specific key."""
+        self._cache.setdefault(cache_type, {})[key] = {
+            "timestamp": datetime.utcnow(),
+            "value": value,
+        }
 
-class AsusWrtCommunicationError(AsusWrtError, aiohttp.ClientError):
-    """Error occurred while communicating with the AsusWrt device ."""
-
-
-class AsusWrtResponseError(AsusWrtCommunicationError):
-    """HTTP error code returned by the AsusWrt device."""
-
-    def __init__(
-        self,
-        *args: Any,
-        status: int,
-        headers: Optional[aiohttp.typedefs.LooseHeaders] = None,
-        message: Optional[str] = None,
-        **kwargs: Any,
-    ) -> None:
-        """Initialize."""
-        if not message:
-            message = f"Did not receive HTTP 200 but {status}"
-        super().__init__(*args, message=message, **kwargs)
-        self.status = status
-        self.headers = headers
-
-
-class AsusWrtClientResponseError(aiohttp.ClientResponseError, AsusWrtResponseError):
-    """HTTP response error with more details from aiohttp."""
-
-
-class AsusWrtConnectionError(AsusWrtCommunicationError, aiohttp.ClientConnectionError):
-    """Error connecting with the router."""
-
-
-class AsusWrtConnectionTimeoutError(
-    AsusWrtCommunicationError, aiohttp.ServerTimeoutError, asyncio.TimeoutError
-):
-    """Timeout while communicating with the device."""
-
-
-class AsusWrtLoginError(AsusWrtError):
-    """Login error / invalid credential."""
-
-
-class AsusWrtValueError(AsusWrtError, ValueError):
-    """Error invalid value received."""
+    def get_key(self, cache_type: str, key: str) -> Any:
+        """Add a value in the cache for specific key."""
+        if cache_type not in self._cache:
+            return None
+        if (cache_value := self._cache[cache_type].get(key)) is None:
+            return None
+        diff = datetime.utcnow() - cache_value["timestamp"]
+        if diff.total_seconds() > self._validity:
+            self._cache[cache_type].pop(key)
+            return None
+        return cache_value["value"]
 
 
 class AsusWrtHttp:
@@ -194,11 +219,25 @@ class AsusWrtHttp:
             self._session = None
             self._managed_session = True
 
-        self._mac = None
+        # initialize 5 seconds cache to avoid
+        # multiple requests in specific interval
+        self._cache = AsusWrtCache(5)
+
+        self._mac: str | None = None
+        self._model: str | None = None
+        self._firmware: AsusWrtFirmware | None = None
+        self._last_fw_check = datetime.utcnow()
+        self._mesh_nodes = None
+
+        # Transfer rate variable
         self._latest_transfer_data = None
         self._latest_transfer_rate = {"rx_rate": 0.0, "tx_rate": 0.0}
         self._latest_transfer_check = None
 
+        # CPU usage variable
+        self._available_cpu = None
+        self._latest_cpu_data = None
+
     def __url(self, path):
         """Return the url to a specific path."""
         return f"{self._protocol}://{self._hostname}:{self._port}/{path}"
@@ -240,14 +279,17 @@ class AsusWrtHttp:
 
         return result
 
-    async def __post(self, command, path=ASUSWRT_GET_PATH, *, retry=True):
+    async def __post(
+        self, *, path=_ASUSWRT_GET_PATH, command: str | None = None, retry=True
+    ):
         """
-        Private POST method to execute a hook on the router and return the result
+        Private POST method to execute a command on the router and return the result
 
-        :param command: Command to send to the return
+        :param path: Path to send to the command
+        :param command: Command to send
         :returns: string result from the router
         """
-        payload = f"hook={command}"
+        payload = command or ""
         try:
             await self.async_connect()
             result = await self.__http_post(
@@ -256,26 +298,66 @@ class AsusWrtHttp:
 
         except (AsusWrtConnectionError, AsusWrtClientResponseError):
             if retry:
-                return await self.__post(command, path, retry=False)
+                return await self.__post(path=path, command=command, retry=False)
             raise
 
-        if result.find(ASUSWRT_ERROR_KEY, 0, len(ASUSWRT_ERROR_KEY) + 5) >= 0:
+        if result.find(_ASUSWRT_ERROR_KEY, 0, len(_ASUSWRT_ERROR_KEY) + 5) >= 0:
             self._auth_headers = None
             if retry:
-                return await self.__post(command, path, retry=False)
+                return await self.__post(path=path, command=command, retry=False)
             raise AsusWrtConnectionError("Not connected to the router")
 
         return result
 
-    async def __send_cmd(self, commands: dict[str, str], action_mode: str = "apply"):
-        """Command device to run a service or set parameter"""
+    async def __send_cmd(
+        self,
+        *,
+        path=_ASUSWRT_CMD_PATH,
+        commands: dict[str, str] | None = None,
+        action_mode: str = "apply",
+    ):
+        """Command device to run a service or set parameter."""
+        add_req = commands or {}
         request: dict = {
-            "action_mode": action_mode,
-            **commands,
+            _ASUSWRT_ACTION_KEY: action_mode,
+            **add_req,
         }
-        result = await self.__post(str(request), ASUSWRT_CMD_PATH)
+        return await self.__post(path=path, command=str(request))
+
+    async def __send_req(self, command: str):
+        """Send a hook request to the device.
+
+        :param command: Command to send
+        :returns: string result from the router
+        """
+        if value := self._cache.get_key(_ASUSWRT_HOOK_KEY, command):
+            return value
+        request = f"{_ASUSWRT_HOOK_KEY}={command}"
+        result = await self.__post(command=request)
+        self._cache.set_key(_ASUSWRT_HOOK_KEY, command, result)
         return result
 
+    async def _run_service(
+        self, service: str, *, arguments: dict[str, str] | None = None
+    ) -> bool:
+        """Command device to run a service.
+
+        :param service: Service to run
+        :param arguments: Arguments for the service to run (optional)
+        :returns: True or False
+        """
+        commands = {_ASUSWRT_SVC_REQ: service}
+        if arguments:
+            commands.update(arguments)
+
+        s = await self.__send_cmd(commands=commands)
+        result = _get_json_result(s)
+        if not all(v in result for v in [_ASUSWRT_SVC_REPLY, _ASUSWRT_SVC_MODIFY]):
+            return False
+        if result[_ASUSWRT_SVC_REPLY] != service:
+            return False
+        return True
+
     @property
     def hostname(self) -> str:
         """Return the device hostname."""
@@ -286,6 +368,18 @@ class AsusWrtHttp:
         """Return the device mac address."""
         return self._mac
 
+    @property
+    def model(self) -> str | None:
+        """Return the device mac address."""
+        return self._model
+
+    @property
+    def firmware(self) -> str | None:
+        """Return the device firmware."""
+        if self._firmware:
+            return self._firmware.to_str()
+        return None
+
     @property
     def is_connected(self) -> bool:
         """Return if connection is active."""
@@ -309,32 +403,107 @@ class AsusWrtHttp:
         auth = f"{self._username}:{self._password}".encode("ascii")
         login_token = base64.b64encode(auth).decode("ascii")
         payload = f"login_authorization={login_token}"
-        headers = {"user-agent": ASUSWRT_USR_AGENT}
+        headers = {"user-agent": _ASUSWRT_USR_AGENT}
 
         result = await self.__http_post(
-            self.__url(ASUSWRT_LOGIN_PATH), headers, payload, get_json=True
+            self.__url(_ASUSWRT_LOGIN_PATH), headers, payload, get_json=True
         )
-        if ASUSWRT_TOKEN_KEY not in result:
+        if _ASUSWRT_TOKEN_KEY not in result:
             raise AsusWrtLoginError("Login Failed")
 
-        token = result[ASUSWRT_TOKEN_KEY]
+        token = result[_ASUSWRT_TOKEN_KEY]
         self._auth_headers = {
-            "user-agent": ASUSWRT_USR_AGENT,
-            "cookie": f"{ASUSWRT_TOKEN_KEY}={token}",
+            "user-agent": _ASUSWRT_USR_AGENT,
+            "cookie": f"{_ASUSWRT_TOKEN_KEY}={token}",
         }
 
         # try to get the main properties after connect
         await self._load_props()
 
-    async def _load_props(self) -> str | None:
+    async def _load_props(self) -> None:
         """Load device properties from NVRam."""
-        if self._mac is not None:
-            return
+        # mac address
+        if self._mac is None:
+            try:
+                result = await self.async_get_settings(_PROP_MAC_ADDR)
+            except AsusWrtError:
+                _LOGGER.debug("Failed to retrieve device mac address")
+            else:
+                self._mac = result.get(_PROP_MAC_ADDR)
+        # model
+        if self._model is None:
+            try:
+                result = await self.async_get_settings(_PROP_MODEL)
+            except AsusWrtError:
+                _LOGGER.debug("Failed to retrieve device model")
+            else:
+                self._model = result.get(_PROP_MODEL)
+        # firmware
         try:
-            result = await self.async_get_settings(PROP_MAC_ADDR)
+            await self.async_get_cur_fw()
         except AsusWrtError:
+            _LOGGER.debug("Failed to retrieve installed firmware")
+
+    async def async_get_cur_fw(self) -> str | None:
+        """Get current device firmware information."""
+        version = build = extend = None
+        if firmver := await self.async_get_settings("firmver"):
+            version = firmver.get("firmver")
+        buildno = await self.async_get_settings("buildno")
+        if buildno and "buildno" in buildno:
+            build = buildno["buildno"]
+            if extendno := await self.async_get_settings("extendno"):
+                extend = extendno.get("extendno")
+        if build:
+            self._firmware = AsusWrtFirmware(version, build, extend)
+        return self.firmware
+
+    async def async_get_new_fw(self) -> str | None:
+        """Get new device firmware available."""
+        try:
+            await self.async_check_fw_update()
+            if not await self.async_get_cur_fw():
+                return None
+            res = await self.__post(path=_ASUSWRT_FW_PATH)
+        except AsusWrtError as ex:
+            _LOGGER.debug("Failed checking for new fw version: %s", ex)
+            return None
+
+        if not (new_ver := _parse_fw_info(res)):
+            return None
+
+        return self._firmware.check_new(new_ver)
+
+    async def async_check_fw_update(self):
+        """Check for firmware update."""
+        call_time = datetime.utcnow()
+        if (call_time - self._last_fw_check).total_seconds() < FW_CHECK_INTERVAL:
             return
-        self._mac = result.get(PROP_MAC_ADDR)
+        self._last_fw_check = call_time
+        try:
+            await self.__send_cmd(path=_ASUSWRT_APPLY_PATH, action_mode=_CMD_FW_CHECK)
+        except AsusWrtError:
+            _LOGGER.debug("Failed to check for new firmware")
+
+    async def async_reboot(self) -> bool:
+        """Reboot the router."""
+        return await self._run_service(_CMD_REBOOT)
+
+    async def async_get_led_status(self) -> bool:
+        """Get device led status."""
+        result = await self.async_get_settings(_PROP_LED_STATUS)
+        try:
+            led = int(result.get(_PROP_LED_STATUS, 0))
+        except (TypeError, ValueError):
+            return False
+        return led != 0
+
+    async def async_set_led_status(self, status: bool) -> bool:
+        """Set device led status."""
+        arguments = {
+            _PROP_LED_STATUS: 1 if status else 0,
+        }
+        return await self._run_service(_CMD_LED_STATUS, arguments=arguments)
 
     async def async_get_uptime(self):
         """
@@ -344,10 +513,10 @@ class AsusWrtHttp:
 
         :returns: JSON with last boot time and uptime in seconds
         """
-        r = await self.__post(f"{CMD_UPTIME}()")
-        since = r.partition(":")[2].partition("(")[0]
+        r = await self.__send_req(f"{_CMD_UPTIME}()")
+        time = r.partition(":")[2].partition("(")[0]
         up = int(r.partition("(")[2].partition(" ")[0])
-        return {"since": since, "uptime": up}
+        return {"uptime": up, "time": time}
 
     async def async_get_memory_usage(self):
         """
@@ -357,22 +526,82 @@ class AsusWrtHttp:
 
         :returns: JSON with memory variables
         """
-        s = await self.__post(f"{CMD_MEMORY_USAGE}({PARAM_APPOBJ})")
-        result = _get_json_result(s, CMD_MEMORY_USAGE)
-        return {k: int(v) for k, v in result.items()}
+        s = await self.__send_req(f"{_CMD_MEMORY_USAGE}({_PARAM_APPOBJ})")
+        result = _get_json_result(s, _CMD_MEMORY_USAGE)
+        result_val = {k: int(v) for k, v in result.items()}
+
+        # calculate memory usage percentage
+        try:
+            mem_usage = round(
+                (result_val["mem_used"] / result_val["mem_total"]) * 100, 2
+            )
+        except (KeyError, TypeError, ValueError, ZeroDivisionError):
+            mem_usage = None
+
+        return {"mem_usage_perc": mem_usage, **result_val}
 
     async def async_get_cpu_usage(self):
         """
         Return CPUs usage of the router
+        Note that at least 2 calls is required to have valid data
 
-        Format: {'cpu1_total': 38106047, 'cpu1_usage': 3395512,
-                 'cpu2_total': 38106008, 'cpu2_usage': 2384694, ...}
+        Format: {'cpu1': 0.22, 'cpu2': 0.01, ... 'cpu_total': 0.21}
 
-        :returns: JSON with CPUs load statistics
+        :returns: JSON with CPUs load percentage
         """
-        s = await self.__post(f"{CMD_CPU_USAGE}({PARAM_APPOBJ})")
-        result = _get_json_result(s, CMD_CPU_USAGE)
-        return {k: int(v) for k, v in result.items()}
+        if self._available_cpu is not None:
+            if not self._available_cpu:
+                return {}
+
+        s = await self.__send_req(f"{_CMD_CPU_USAGE}({_PARAM_APPOBJ})")
+        result = _get_json_result(s, _CMD_CPU_USAGE)
+
+        cpu_data = {}
+        for key, val in result.items():
+            if not key.startswith("cpu"):
+                continue
+            cpu_info = key.split("_")
+            if len(cpu_info) != 2:
+                continue
+            cpu_data.setdefault(f"{cpu_info[0]}_usage", {})[cpu_info[1]] = int(val)
+
+        if self._available_cpu is None:
+            self._available_cpu = [k for k in cpu_data]
+            if not self._available_cpu:
+                return {}
+
+        # calculate the CPU usage
+        prev_cpu_data = self._latest_cpu_data or {}
+        cpu_usage = {}
+        for key in self._available_cpu:
+            if not (key in cpu_data and key in prev_cpu_data):
+                cpu_usage[key] = 0.0
+                continue
+            cpu_usage[key] = _calculate_cpu_usage(cpu_data[key], prev_cpu_data[key])
+
+        # calculate the total CPU average usage
+        cpu_avg = [v for v in cpu_usage.values()]
+        cpu_usage["cpu_total_usage"] = round(sum(cpu_avg) / len(cpu_avg), 2)
+
+        # save last fetched data
+        self._latest_cpu_data = cpu_data.copy()
+
+        return cpu_usage
+
+    async def async_get_temperatures(self):
+        """
+        Return Temperatures from the router
+
+        Format: {'2.4GHz': 42.0, '5.0GHz': 48.0, 'CPU': 64.0, ...}
+
+        :returns: JSON with Temperatures statistics
+        """
+        if result := self._cache.get_key(_CACHE_SPECIFIC_URI, _ASUSWRT_TEMP_PATH):
+            return result
+        s = await self.__post(path=_ASUSWRT_TEMP_PATH)
+        result = _parse_temperatures(s)
+        self._cache.set_key(_CACHE_SPECIFIC_URI, _ASUSWRT_TEMP_PATH, result)
+        return result
 
     async def async_get_wan_info(self):
         """
@@ -386,11 +615,11 @@ class AsusWrtHttp:
 
         :returns: JSON with status information on the WAN connection
         """
-        r = await self.__post(f"{CMD_WAN_INFO}()")
+        r = await self.__send_req(f"{_CMD_WAN_INFO}()")
         status = {}
         for f in r.split("\n"):
             if "return" in f:
-                if f"{CMD_WAN_INFO}_" in f:
+                if f"{_CMD_WAN_INFO}_" in f:
                     key = f.partition("(")[0].partition("_")[2]
                     value = (f.rpartition(" ")[-1][:-2]).replace("'", "")
                     status[key] = value
@@ -413,8 +642,8 @@ class AsusWrtHttp:
 
         :returns: JSON with a list of DHCP leases
         """
-        s = await self.__post(f"{CMD_DHCP_LEASE}()")
-        return _get_json_result(s, CMD_DHCP_LEASE)
+        s = await self.__send_req(f"{_CMD_DHCP_LEASE}()")
+        return _get_json_result(s, _CMD_DHCP_LEASE)
 
     async def async_get_traffic_bytes(self):
         """
@@ -424,19 +653,20 @@ class AsusWrtHttp:
 
         :returns: JSON with sent and received bytes since last boot
         """
-        s = await self.__post(f"{CMD_NET_TRAFFIC}({PARAM_APPOBJ})")
-        meas = _get_json_result(s, CMD_NET_TRAFFIC)
-        traffic = None
+        s = await self.__send_req(f"{_CMD_NET_TRAFFIC}({_PARAM_APPOBJ})")
+        meas = _get_json_result(s, _CMD_NET_TRAFFIC)
         if "INTERNET_rx" in meas:
-            traffic = "INTERNET"
-        elif "BRIDGE_rx" in meas:
-            traffic = "BRIDGE"
-
-        if traffic:
-            rx = int(meas[f"{traffic}_rx"], base=16)
-            tx = int(meas[f"{traffic}_tx"], base=16)
+            traffics = ["INTERNET"]
         else:
-            rx = tx = 0
+            traffics = ["WIRED", "WIRELESS0", "WIRELESS1"]
+        # elif "BRIDGE_rx" in meas:
+        #     traffics = ["BRIDGE"]
+
+        rx = tx = 0
+        for traffic in traffics:
+            if f"{traffic}_rx" in meas:
+                rx += int(meas[f"{traffic}_rx"], base=16)
+                tx += int(meas[f"{traffic}_tx"], base=16)
 
         return {"rx": rx, "tx": tx}
 
@@ -493,10 +723,10 @@ class AsusWrtHttp:
         :param setting: the setting name to query (leave empty to get all main settings)
         :returns: JSON with main Router settings or specific one
         """
-        setting_list = [setting] if setting else NVRAM_INFO
+        setting_list = [setting] if setting else _NVRAM_INFO
         result = {}
         for s in setting_list:
-            resp = await self.__post(_nvram_cmd(s))
+            resp = await self.__send_req(_nvram_cmd(s))
             if resp:
                 result[s] = _get_json_result(resp, s)
         return result
@@ -524,9 +754,9 @@ class AsusWrtHttp:
                 ]
         :returns: JSON with list of mac address and all client related info
         """
-        s = await self.__post(f"{CMD_CLIENT_LIST}()")
+        s = await self.__send_req(f"{_CMD_CLIENT_LIST}()")
         result = _get_json_result(s)
-        return [result.get(CMD_CLIENT_LIST, {})]
+        return [result.get(_CMD_CLIENT_LIST, {})]
 
     async def async_get_connected_mac(self):
         """
@@ -555,8 +785,9 @@ class AsusWrtHttp:
         mesh_nodes = {self._mac: self._hostname} if self._mac else {}
         for mac, info in clnts[0].items():
             if len(mac) == 17 and info.get("isOnline", "0") == "1":
-                if is_mesh_node := info.get("amesh_isRe", "0") == "1":
+                if info.get("amesh_isRe", "0") == "1":
                     mesh_nodes[mac] = info.get("ip")
+                    continue
                 if not (name := info.get("nickName")):
                     name = info.get("name")
                 is_wl = info.get("isWL", "0") != "0"
@@ -566,11 +797,11 @@ class AsusWrtHttp:
                         info.get("ip"),
                         name,
                         info.get("amesh_papMac"),
-                        is_mesh_node,
                         is_wl,
                     )
                 )
 
+        self._mesh_nodes = mesh_nodes
         result = {}
         for dev in dev_list:
             node_ip = mesh_nodes.get(dev.node) if dev.node else None
@@ -579,12 +810,23 @@ class AsusWrtHttp:
                 dev.ip,
                 dev.name,
                 node_ip or self._hostname,
-                dev.is_mesh_node,
                 dev.is_wl,
             )
 
         return result
 
+    async def async_get_mesh_nodes(self):
+        """
+        Return a list of available mesh nodes
+
+        Format: {"AC:84:C6:6C:A7:C0": "x.x.x.x"}, ...}
+        :return: JSON dict with mac as key and ip address as value
+        """
+        if self._mesh_nodes is None:
+            await self.async_get_connected_devices()
+
+        return self._mesh_nodes
+
     async def async_get_client_info(self, client_mac):
         """
         Get info on a single client
